import org.junit.Test;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.logging.FileHandler;
import java.util.logging.Handler;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.logging.SimpleFormatter;

public class ExecuteBatchTest {
    // JDBC driver name and database URL
    static final String JDBC_DRIVER = "com.microsoft.sqlserver.jdbc.SQLServerDriver";
    static final String DB_NAME = "DBNAME";
    static final String DB_URL = "jdbc:sqlserver://127.0.0.1:1433;databaseName=" + DB_NAME;

    //  Database credentials
    static final String USER = "DBUSER";
    static final String PASS = "DBPASSWORD";

    private static final String CREATE_TABLE_SQL = "create table TEST2 (KEY1 numeric(19,0) not null, KEY2 numeric(19,0) not null, primary key (KEY1, KEY2))";
    private static final String INSERT_ROW_SQL = "INSERT INTO TEST2(KEY1, KEY2) VALUES(?, ?)";

    @Test
    public void testWithColumnEncryption() throws Exception {
        Logger logger = Logger.getLogger("com.microsoft.sqlserver.jdbc.Statement");
        logger.setLevel(Level.FINER);
        Handler fh = new FileHandler("/tmp/mssql-jdbc.trace");
        logger.addHandler(fh);
        SimpleFormatter formatter = new SimpleFormatter();
        fh.setFormatter(formatter);

        test(DB_URL + ";columnEncryptionSetting=Enabled");
    }

    @Test
    public void testWithoutColumnEncryption() throws Exception {
        test(DB_URL);
    }

    private void test(String url) throws Exception {
        try (Connection conn = getConnection(JDBC_DRIVER, url, USER, PASS)) {
            try (PreparedStatement pstmt = conn.prepareStatement(INSERT_ROW_SQL)) {
                createTable(conn);

                // Set auto-commit to false
                conn.setAutoCommit(false);

                executeBatch(pstmt, 10, url.hashCode() + 1);

                pstmt.clearParameters();

                executeBatch(pstmt, 10, url.hashCode() + 2);

                conn.commit();
            } catch (Exception e) {
                conn.rollback();
                throw e;
            }
        }
    }

    private void createTable(Connection conn) {
        try (Statement stmt = conn.createStatement()) {
            stmt.execute(CREATE_TABLE_SQL);
        } catch (SQLException e) {
            System.out.println("Error when creating table: " + e.getMessage());
        }
    }

    private void executeBatch(PreparedStatement pstmt, int count, int run) throws SQLException {
        long base = System.currentTimeMillis();

        for (int idx = 1; idx <= count; idx++) {
            pstmt.setLong(1, base + idx);
            pstmt.setLong(2, run);
            pstmt.addBatch();
        }

        int[] rowCounts = pstmt.executeBatch();
        for (int idx = 0; idx < rowCounts.length; idx++) {
            System.out.printf("Row %2d %s successfully inserted.\n", idx + 1, rowCounts[idx] == 1 ? "was" : "was not");
        }
    }

    private Connection getConnection(String jdbcDriver, String dbUrl, String user, String pass) throws Exception {
        Class.forName(jdbcDriver);

        System.out.println("Connecting to database...");
        return DriverManager.getConnection(dbUrl, user, pass);
    }
}
